#==================================================================
#  
#  TRACEO: Barents flat waveguide
#  Faro, Sab Set 11 22:20:46 WEST 2021
#  Written by Orlando Camargo Rodriguez
#  
#==================================================================

from os import system
import sys
sys.path.append("../../../")
from numpy import *
from scipy.io import *
from matplotlib.pyplot import *
from wtraceoinfil import *

system("rm -rf ari.mat")

case_title = '''Barents Ray Trace'''

#==================================================================
#  
#  Define source data:
#  
#==================================================================

freq   = 224
Rmaxkm = 30
Rmax   = Rmaxkm*1000
Dmax   = 300

ray_step = 10

zs =   50.0
rs =    0.0
thetamax = 14
nthetas =  51
thetas = linspace(-thetamax, thetamax, nthetas)
position = array([rs,zs])
rbox     = array([rs-1,Rmax+1])

source_data = {"ds":ray_step, "position":position, "rbox":rbox,"f":freq,"thetas":thetas}

#==================================================================
#  
#  Define altimetry data:
#  
#==================================================================

altimetry  = array([[rs-2,Rmax+2],[0,0]])
properties = array([0,0,0,0,0])
btype = "V"
ptype = "H"
units = "W"
itype = "FL"

surface_data = {"type":btype, "ptype":ptype, "units":units,"itype":itype,"x":altimetry,"properties":properties}

#==================================================================
#  
#  Define sound speed data:
#  
#==================================================================

ranges = 0.0
# McLaughlin1992, p.11: 
z0 = array([   0,   10,   50,   75,  100  ,  150,  200,  250  ,  300])
c0 = array([1481, 1480, 1470, 1467, 1466.5, 1466, 1465, 1464.5, 1463])
n = 4
coefs = polyfit(z0, c0, n)
polyc  = poly1d(coefs)
depths = linspace(0,300,301)
c      = polyc( depths )

ssp_data = {"cdist":"c(z,z)","cclass":"TABL","z":depths,"r":ranges,"c":c}

#==================================================================
#  
#  Define object data:
#  
#==================================================================

npo = array([0])

robj = 0
zup  = 0
zdn  = 0

x = array( [robj,zup,zdn] )
 
xobj = array( [ x ] ) 

properties = [0,0,0,0,0]
oproperties = array([properties])

btype = "R"
units = "W"
itype = "2P"

object_data = {"nobjects":0,"itype":itype,"type":btype,"x":xobj,"units":units,"properties":oproperties,"npobjects":npo}

#==================================================================
#  
#  Define bathymetry data:
#  
#==================================================================

bathymetry = array([[rs-2,Rmax+2],[Dmax,Dmax]])

properties = array([1550.0,600.0,2.0,0.1,0.0])
btype = "E"
ptype = "H"
units = "W"
itype = "FL"

bottom_data = {"type":btype, "ptype":ptype, "units":units,"itype":itype,"x":bathymetry,"properties":properties}

#==================================================================
#  
#  Define output data:
#  
#==================================================================

rarray = array( [Rmax] )
zarray = array( [Dmax/2] )

miss = 0.0

output_data = {"ctype":"ARI","array_shape":"RRY","r":rarray,"z":zarray,"miss":miss}

#==================================================================
#  
#  Call the function:
#  
#==================================================================

print('Writing TRACEO waveguide input file...')

wtraceoinfil("barents.in",case_title,source_data,surface_data,ssp_data,object_data,bottom_data,output_data)

print('Calling TRACEO...')

system('traceo.exe barents.in')

print('Reading the output data...')

data = loadmat('ari.mat')

rayname = 'ray00000'

thetas = data["ray_elev"]

nthetas = thetas.size

tau = zeros( nthetas )
thetad = zeros( nthetas )

figure(1)
hold(True)
for i in range(1,nthetas+1):

   if i < 10:
      rayname = 'ray0000' + str(i)
   elif i < 100:
      rayname = 'ray000'  + str(i)
   elif i < 1000:
      rayname = 'ray00'   + str(i)
   elif i < 10000:
      rayname = 'ray0'    + str(i)
   else:
      rayname = 'ray'     + str(i)   

   rayi = data[rayname]
   
   r =  rayi[0,]; rkm = r/1000
   z = -rayi[1,]
   t =  rayi[2,]
   tau[i-1] = t[-1]
   dr = r[-1] - r[-2]
   dz = z[-1] - z[-2]
   thetad[i-1] = -arctan2(dz,dr)*189.0/pi
   plot(rkm,z)

n = 2
coefs = polyfit(thetad, tau, n)
polytau  = poly1d(coefs)
sthetad = sort( thetad )
print( coefs )

xlabel('Range (km)')
ylabel('Depth (m)')
title('TRACEO - Barents ray trace')   
xlim(0,Rmaxkm)
grid(True)

figure(2)
plot(tau,thetad,'o')
plot(polytau(sthetad),sthetad,'b',linewidth=2)
xlabel(r'$\tau$ (s)',fontsize=16)
ylabel(r'$\theta$ (Degrees)',fontsize=16)
grid(True)

show()

print('done.')
