#==================================================================
#  
#  TRACEO: Munk flat waveguide
#  Faro, Sab Set 11 22:14:49 WEST 2021
#  Written by Orlando Camargo Rodriguez
#  
#==================================================================

from os import *
import sys
sys.path.append ("../../../")
from wtraceoinfil import *
from numpy import *
from scipy.io import *
from matplotlib.pyplot import *

system("rm -rf cpr.mat")

case_title = '''PAM_WMXZ'''

#==================================================================
#  
#  Define source data:
#  
#==================================================================

freq   = 40000;
Rmaxkm = 6.1 
Rmax = Rmaxkm*1000

sspdata = loadtxt("ssp.dat")

z = sspdata[:,0]; Dmax = max( z )
c = sspdata[:,1]

ray_step = 50

zs = 600.0
rs =   0.0
thetamax =  89
nthetas =  179
thetas = linspace(-thetamax, thetamax, nthetas)
position = array([rs,zs])
rbox     = array([rs-1,Rmax+1])

source_data = {"ds":ray_step, "position":position, "rbox":rbox,"f":freq,"thetas":thetas}

#==================================================================
#  
#  Define altimetry data:
#  
#==================================================================

altimetry  = array([[rs-2,Rmax+2],[0,0]])
properties = array([0,0,0,0,0])
btype = "V"
ptype = "H"
units = "W"
itype = "FL"

surface_data = {"type":btype, "ptype":ptype, "units":units,"itype":itype,"x":altimetry,"properties":properties}

#==================================================================
#  
#  Define sound speed data:
#  
#==================================================================

ssp_data = {"cdist":"c(z,z)","cclass":"TABL","z":z,"r":array([0]),"c":c}

#==================================================================
#  
#  Define object data:
#  
#==================================================================

npo = array([0])

robj = 0
zup  = 0
zdn  = 0

x = array( [robj,zup,zdn] )
 
xobj = array( [ x ] ) 

properties = [0,0,0,0,0]
oproperties = array([properties])

btype = "R"
units = "W"
itype = "2P"

object_data = {"nobjects":0,"itype":itype,"type":btype,"x":xobj,"units":units,"properties":oproperties,"npobjects":npo}

#==================================================================
#  
#  Define bathymetry data:
#  
#==================================================================

bathymetry = array([[rs-2,Rmax+2],[Dmax,Dmax]])

properties = array([3000.0,0.0,2.4,0.1,0.0])
btype = "E"
ptype = "H"
units = "W"
itype = "FL"

bottom_data = {"type":btype, "ptype":ptype, "units":units,"itype":itype,"x":bathymetry,"properties":properties}

#==================================================================
#  
#  Define output data:
#  
#==================================================================

nra = 501; rarray = array( linspace(0,Rmax,nra) )
nza = 501; zarray = array( linspace(0,2500,nza) )

miss = 0.5

output_data = {"ctype":"CPR","array_shape":"RRY","r":rarray,"z":zarray,"miss":miss}

#==================================================================
#  
#  Call the function:
#  
#==================================================================

print('Writing TRACEO waveguide input file...')

wtraceoinfil("pam.in",case_title,source_data,surface_data,ssp_data,object_data,bottom_data,output_data)

print('Calling TRACEO...')

system('traceo.exe pam.in');

print('Reading the output data...')

data = loadmat('cpr.mat')

pressure = data["pressure"]
pressure = where( pressure == 0, nan, pressure )

tl = 20*log10( abs( pressure ) );

figure(1)
imshow(tl,extent=[0,Rmax,-2500,0], aspect='auto',vmin=-150,vmax=0)
plot(rs,-zs,marker="<",markersize=16,color="k")
plot(bathymetry[0,],-bathymetry[1,],color='k',linewidth=2)
colorbar()
xlim(0,Rmax)
ylim(-2500,0)
xlabel('Range (m)')
ylabel('Depth (m)')
title('TRACEO - PAM calculations, Chapter 3')

show()

print('done.')
