#==================================================================
#  
#  TRACEO: comparison agains 3DWAPE (cross-slope propagation)
#  Faro, Sab Set 11 22:24:45 WEST 2021
#  Written by Orlando Camargo Rodriguez
#  
#==================================================================

from os import *
import sys
sys.path.append ("../../../")
from wtraceoinfil import *
from numpy import *
from scipy.io import *
from matplotlib.pyplot import *

system("rm -rf cpr.mat")

case_title = '''Cross-slope propagation''';

refdata = loadtxt("tlr_across_slope_RD30m.dat")

rref  = refdata[:,0]
tlref = refdata[:,1]

#==================================================================
#  
#  Define source data:
#  
#==================================================================

freq   = 25
Rmaxkm = 25
Rmax   = Rmaxkm*1000
Dmax   = 200

ray_step = 10.0

zs = 100.0
rs =   0.0
thetamax =  60
nthetas = 1001
thetas = linspace(-thetamax, thetamax, nthetas)
position = array([rs,zs])
rbox     = array([rs-1,Rmax+1])

source_data = {"ds":ray_step, "position":position, "rbox":rbox,"f":freq,"thetas":thetas}

#==================================================================
#  
#  Define altimetry data:
#  
#==================================================================

altimetry  = array([[rs-2,Rmax+2],[0,0]])
properties = array([0,0,0,0,0])
btype = "V"
ptype = "H"
units = "W"
itype = "FL"

surface_data = {"type":btype, "ptype":ptype, "units":units,"itype":itype,"x":altimetry,"properties":properties}

#==================================================================
#  
#  Define sound speed data:
#  
#==================================================================

c0 = 1500

ssp_data = {"cdist":"c(z,z)","cclass":"ISOV","z":array([0,Dmax]),"r":array([0]),"c":array([c0,c0])}

#==================================================================
#  
#  Define object data:
#  
#==================================================================

npo = array([0])

robj = 0
zup  = 0
zdn  = 0

x = array( [robj,zup,zdn] )
 
xobj = array( [ x ] ) 

properties = [0,0,0,0,0]
oproperties = array([properties])

btype = "R"
units = "W"
itype = "2P"

object_data = {"nobjects":0,"itype":itype,"type":btype,"x":xobj,"units":units,"properties":oproperties,"npobjects":npo}

#==================================================================
#  
#  Define bathymetry data:
#  
#==================================================================

bathymetry = array([[rs-2,Rmax+2],[Dmax,Dmax]])

properties = array([1700.0,0.0,1.5,0.5,0.0])
btype = "E"
ptype = "H"
units = "W"
itype = "FL"

bottom_data = {"type":btype, "ptype":ptype, "units":units,"itype":itype,"x":bathymetry,"properties":properties}

#==================================================================
#  
#  Define output data:
#  
#==================================================================

rarray = rref
zarray = array([ 30.0 ])

miss = 0.5

output_data = {"ctype":"CPR","array_shape":"HRY","r":rarray,"z":zarray,"miss":miss}

#==================================================================
#  
#  Call the function:
#  
#==================================================================

print('Writing TRACEO waveguide input file...')

wtraceoinfil("3dwape.in",case_title,source_data,surface_data,ssp_data,object_data,bottom_data,output_data)

print('Calling TRACEO...')

system('traceo.exe 3dwape.in');

print('Reading the output data...')

data = loadmat('cpr.mat')

pressure = data["pressure"]
pressure = where( pressure == 0, nan, pressure )
tl = 20*log10( abs( pressure ) )

figure(1)
plot(rref/1000,tlref,'k',lineWidth=2)
plot(rref/1000,tl,'k--',linewidth=2)
grid(True)
xlabel('Range (km)')
ylabel('TL (dB)')
title('TRACEO vs. 3DWAPE @ 25 Hz')

show()

print('done.')
