#==================================================================
#  
#  TRACEO: CALCOM ray test
#  Faro, Sab Set 11 22:23:36 WEST 2021
#  Written by Orlando Camargo Rodriguez 
#  
#==================================================================

from os import *
import sys
sys.path.append ("../../../")
from wtraceoinfil import *
from math import *
from numpy import *
from scipy.io import *
from matplotlib.pyplot import *

system("rm -rf ari.mat")

case_title = '''CALCOM waveguide''';

data = loadmat('ssp.mat')

z0 = squeeze( data['z2'] )
c0 = squeeze( data['ssp'] )

Dmax = max( z0 )
#==================================================================
#  
#  Define source data:
#  
#==================================================================

freq   = 500.0
Rmaxkm =  5.5
Rmax   = Rmaxkm*1000

ray_step = 5

zs = 10.0
rs =  0.0
theta1 = 29.0
theta2 = 30.0
nthetas  = 21
thetas = linspace(theta1, theta2, nthetas)
position = array([rs,zs])
rbox     = array([rs-1.0,Rmax+1.0])

source_data = {"ds":ray_step, "position":position, "rbox":rbox,"f":freq,"thetas":thetas}

#==================================================================
#  
#  Define altimetry data:
#  
#==================================================================

altimetry  = array([[rs-2.0,Rmax+2.0],[0.0,0.0]])
properties = array([0,0,0,0,0])
btype = "V"
ptype = "H"
units = "W"
itype = "FL"

surface_data = {"type":btype, "ptype":ptype, "units":units,"itype":itype,"x":altimetry,"properties":properties}

#==================================================================
#  
#  Define sound speed data:
#  
#==================================================================

ssp_data = {"cdist":"c(z,z)","cclass":"TABL","z":z0,"r":array([0]),"c":c0}

#==================================================================
#  
#  Define object data:
#  
#==================================================================

npo   = array([0]) 
ro    = Rmax/2
zo    = Dmax/2

robj  = ro
zup   = zo
zdn   = zo

x = array( [robj,zdn,zup] )
 
xobj = array( [ x ] ) 

properties = [0,0,0,0,0]
oproperties = array([properties])

btype = "R"
units = "W"
itype = "2P"

object_data = {"nobjects":0,"itype":itype,"type":btype,"x":xobj,"units":units,"properties":oproperties,"npobjects":npo}

#==================================================================
#  
#  Define bathymetry data:
#  
#==================================================================

bathymetry = array([[rs-2.0,Rmax+2.0],[Dmax,Dmax]])
properties = array([1750.0,0.0,1.9,0.8,0.0])
btype = "E"
ptype = "H"
units = "W"
itype = "FL"

bottom_data = {"type":btype, "ptype":ptype, "units":units,"itype":itype,"x":bathymetry,"properties":properties}

#==================================================================
#  
#  Define output data:
#  
#==================================================================

rarray = array( [Rmax] )
zarray = array( [10.0] )
miss = 0.5

output_data = {"ctype":"ARI","array_shape":"VRY","r":rarray,"z":zarray,"miss":miss}

#==================================================================
#  
#  Call the function:
#  
#==================================================================

print('Writing TRACEO waveguide input file...')

wtraceoinfil("calcom.in",case_title,source_data,surface_data,ssp_data,object_data,bottom_data,output_data)

print('Calling TRACEO...')

system('traceo.exe calcom.in')

print('Reading the output data...')

data = loadmat('ari.mat')

rayname = 'ray00000'

tau = zeros( nthetas )
amp = zeros( nthetas ) + 1j*zeros( nthetas )
arrival_angle = zeros( nthetas ) 
z_at_R        = zeros( nthetas )

counter = 1
figure(counter)
for i in range(1,nthetas+1):

   if i < 10:
      rayname = 'ray0000' + str(i)
   elif i < 100:
      rayname = 'ray000'  + str(i)
   elif i < 1000:
      rayname = 'ray00'   + str(i)
   elif i < 10000:
      rayname = 'ray0'    + str(i)
   else:
      rayname = 'ray'     + str(i)   

   rayi = data[rayname]
   
   r =  rayi[0,]
   z = -rayi[1,];z_at_R[i-1] = z[-1] 
   t =  rayi[2,]; tau[i-1] = t[-1]
   a =  rayi[3,] + 1j*rayi[4,]; amp[i-1] = a[-1]

   dr = r[-1] - r[-2]
   dz = z[-1] - z[-2]
   arrival_angle[i-1] = -180/pi*atan2(dz,dr)

   plot(r,z)

xlabel('Range (m)')
ylabel('Depth (m)')
title('TRACEO - CALCOM waveguide, eigenrays')   
xlim(0,Rmax)
ylim(-Dmax,0)
grid(True)

counter = counter + 1
figure(counter)
plot(tau,z_at_R)
xlabel('Travel time (s)')
ylabel('Final depth (m)')
grid(True)

show()

print('done.')
