#==================================================================
#  
#  TRACEO: CALCOM eigenray test
#  Faro, Sab Set 11 22:23:09 WEST 2021
#  Written by Orlando Camargo Rodriguez 
#  
#==================================================================

from os import *
import sys
sys.path.append ("../../../")
from wtraceoinfil import *
from math import *
from numpy import *
from scipy.io import *
from matplotlib.pyplot import *

system("rm -rf eig.mat")

case_title = '''CALCOM waveguide''';

data = loadmat('ssp.mat')

z0 = squeeze( data['z2'] )
c0 = squeeze( data['ssp'] )

Dmax = max( z0 )

nz = 101
zi = linspace(0.0,Dmax,nz)
ci = interp(zi, z0, c0)

hyds = array([10.0,14.0,18.0,22.0,26.0,30.0,34.0,38.0,42.0,46.0,50.0,54.0,58.0,62.0,66.0,70.0,80.0])

#==================================================================
#  
#  Define source data:
#  
#==================================================================

freq   = 500.0
Rmaxkm =  5.5
Rmax   = Rmaxkm*1000

ray_step = 5

zs = 10.0
rs =  0.0
theta1 = -45.0
theta2 = 45.0
nthetas  = 8001
thetas = linspace(theta1, theta2, nthetas)

position = array([rs,zs])
rbox     = array([rs-ray_step,Rmax+ray_step])

source_data = {"ds":ray_step, "position":position, "rbox":rbox,"f":freq,"thetas":thetas}

#==================================================================
#  
#  Define altimetry data:
#  
#==================================================================

altimetry  = array([[rs-2*ray_step,Rmax+2*ray_step],[0.0,0.0]])
properties = array([0,0,0,0,0])
btype = "V"
ptype = "H"
units = "W"
itype = "FL"

surface_data = {"type":btype, "ptype":ptype, "units":units,"itype":itype,"x":altimetry,"properties":properties}

#==================================================================
#  
#  Define sound speed data:
#  
#==================================================================

ssp_data = {"cdist":"c(z,z)","cclass":"TABL","z":z0,"r":array([0]),"c":c0}

#==================================================================
#  
#  Define object data:
#  
#==================================================================

npo   = array([0]) 
ro    = Rmax/2
zo    = Dmax/2

robj  = ro
zup   = zo
zdn   = zo

x = array( [robj,zdn,zup] )
 
xobj = array( [ x ] ) 

properties = [0,0,0,0,0]
oproperties = array([properties])

btype = "R"
units = "W"
itype = "2P"

object_data = {"nobjects":0,"itype":itype,"type":btype,"x":xobj,"units":units,"properties":oproperties,"npobjects":npo}

#==================================================================
#  
#  Define bathymetry data:
#  
#==================================================================

bathymetry = array([[rs-2*ray_step,Rmax+2*ray_step],[Dmax,Dmax]])
properties = array([1750.0,0.0,1.9,0.8,0.0])
btype = "E"
ptype = "H"
units = "W"
itype = "FL"

bottom_data = {"type":btype, "ptype":ptype, "units":units,"itype":itype,"x":bathymetry,"properties":properties}

#==================================================================
#  
#  Define output data:
#  
#==================================================================

rarray = array( [Rmax] )
#zarray = array( [10.0] ) #; print zarray
zarray = array( [hyds[0]] )
miss = 0.5

output_data = {"ctype":"EPR","array_shape":"VRY","r":rarray,"z":zarray,"miss":miss}

#==================================================================
#  
#  Call the function:
#  
#==================================================================

print('Writing TRACEO waveguide input file...')

wtraceoinfil("calcom.in",case_title,source_data,surface_data,ssp_data,object_data,bottom_data,output_data)

print('Calling TRACEO...')

system('traceo.exe calcom.in')

print('Reading the output data...')

data = loadmat('eig.mat')

rayname = 'ray00000'

neigrays = int( squeeze( data["neigrays"] ) )

if neigrays > 0:

   print "Found " + str(neigrays) + " eigenrays..."
   
   tau = zeros( neigrays )
   amp = zeros( neigrays ) + 1j*zeros( neigrays )
   arrival_angle = zeros( neigrays )

   for i in range(1,neigrays+1):

       if i < 10:
          rayname = 'ray0000' + str(i)
       elif i < 100:
          rayname = 'ray000'  + str(i)
       elif i < 1000:
          rayname = 'ray00'   + str(i)
       elif i < 10000:
          rayname = 'ray0'    + str(i)
       else:
          rayname = 'ray'     + str(i)   

       rayi = data[rayname]
   
       r =  rayi[0,]
       z = -rayi[1,]    
       t =  rayi[2,]; tau[i-1] = t[-1]
       a =  rayi[3,] + 1j*rayi[4,]; amp[i-1] = a[-1]

       dr = r[-1] - r[-2]
       dz = z[-1] - z[-2]
       arrival_angle[i-1] = -180/pi*atan2(dz,dr)
       
       figure(1)
       plot(r,z)
   
   figure(1)
   grid(True)
   plot(rs,-zs,'k*',markersize=20)
   plot(rarray,-zarray,'ks',markersize=10)
   xlabel('Range (m)')
   ylabel('Depth (m)')
   title('TRACEO - CALCOM waveguide')
   xlim(0,Rmax)

   figure(2)
   plot(c0,-z0)
   plot(ci,-zi,'o')
   xlabel('Sound Speed (m/s)')
   ylabel('Depth (m)')
   grid(True)

   figure(3)
   stem(tau,abs(amp))
   xlabel('Travel time (s)')
   ylabel('Amplitude')
   grid(True)

   figure(4)
   plot(tau,arrival_angle,'o')
   xlabel('Travel time (s)')
   ylabel('Arrival angle')
   grid(True)

else:
   print "No eigenrays found...."

show()

print('done.')
