#==================================================================
#  
#  TRACEO: INTIMATE eigenray calculations (and arrival angles)
#  Faro, Sab Set 11 22:22:22 WEST 2021
#  Written by Orlando Camargo Rodriguez 
#  
#==================================================================

from os import *
import sys
sys.path.append ("../../../")
from wtraceoinfil import *
from math import *
from numpy import *
from scipy.io import *
from matplotlib.pyplot import *

system("rm -rf eig.mat")

case_title = '''INTIMATE waveguide & eigenrays''';

#==================================================================
#  
#  Define source data:
#  
#==================================================================

freq   = 500.0
Rmaxkm =  5.5
Rmax   = Rmaxkm*1000
Dmax   = 135.0

ray_step = 5

zs = 92.0
rs =  0.0
thetamax = 30
nthetas  = 501
thetas = linspace(-thetamax, thetamax, nthetas)
position = array([rs,zs])
rbox     = array([rs-1.0,Rmax+1.0])

source_data = {"ds":ray_step, "position":position, "rbox":rbox,"f":freq,"thetas":thetas}

#==================================================================
#  
#  Define altimetry data:
#  
#==================================================================

altimetry  = array([[rs-2.0,Rmax+2.0],[0.0,0.0]])
properties = array([0,0,0,0,0])
btype = "V"
ptype = "H"
units = "W"
itype = "FL"

surface_data = {"type":btype, "ptype":ptype, "units":units,"itype":itype,"x":altimetry,"properties":properties}

#==================================================================
#  
#  Define sound speed data:
#  
#==================================================================

data = loadmat('c0.mat')

z = squeeze( data['depths'] )
c = squeeze( data['mean_ssp'] )

ssp_data = {"cdist":"c(z,z)","cclass":"TABL","z":z,"r":array([0]),"c":c}

#==================================================================
#  
#  Define object data:
#  
#==================================================================

npo   = array([0]) 
ro    = Rmax/2
zo    = Dmax/2

robj  = ro
zup   = zo
zdn   = zo

x = array( [robj,zdn,zup] )
 
xobj = array( [ x ] ) 

properties = [0,0,0,0,0]
oproperties = array([properties])

btype = "R"
units = "W"
itype = "2P"

object_data = {"nobjects":0,"itype":itype,"type":btype,"x":xobj,"units":units,"properties":oproperties,"npobjects":npo}

#==================================================================
#  
#  Define bathymetry data:
#  
#==================================================================

bathymetry = array([[rs-2.0,Rmax+2.0],[Dmax,Dmax]])
properties = array([1750.0,0.0,1.9,0.8,0.0])
btype = "E"
ptype = "H"
units = "W"
itype = "FL"

bottom_data = {"type":btype, "ptype":ptype, "units":units,"itype":itype,"x":bathymetry,"properties":properties}

#==================================================================
#  
#  Define output data:
#  
#==================================================================

rarray = array( [Rmax] )
zarray = array( [115.0] )
miss = 0.5

output_data = {"ctype":"ERF","array_shape":"VRY","r":rarray,"z":zarray,"miss":miss}

#==================================================================
#  
#  Call the function:
#  
#==================================================================

print('Writing TRACEO waveguide input file...')

wtraceoinfil("intimate.in",case_title,source_data,surface_data,ssp_data,object_data,bottom_data,output_data)

print('Calling TRACEO...')

system('traceo.exe intimate.in');

print('Reading the output data...')

data = loadmat('eig.mat')

rayname = 'ray00000'

neigrays = int( squeeze( data["neigrays"] ) )

tau = zeros( neigrays )
amp = zeros( neigrays ) + 1j*zeros( neigrays )
arrival_angle = zeros( neigrays ) 

figure(1)
for i in range(1,neigrays+1):

   if i < 10:
      rayname = 'ray0000' + str(i)
   elif i < 100:
      rayname = 'ray000'  + str(i)
   elif i < 1000:
      rayname = 'ray00'   + str(i)
   elif i < 10000:
      rayname = 'ray0'    + str(i)
   else:
      rayname = 'ray'     + str(i)   

   rayi = data[rayname]
   
   r =  rayi[0,]
   z = -rayi[1,]    
   t =  rayi[2,]; tau[i-1] = t[-1]
   a =  rayi[3,] + 1j*rayi[4,]; amp[i-1] = a[-1]

   dr = r[-1] - r[-2]
   dz = z[-1] - z[-2]
   arrival_angle[i-1] = -180/pi*atan2(dz,dr)

   plot(r,z)

xlabel('Range (m)')
ylabel('Depth (m)')
title('TRACEO - INTIMATE waveguide, eigenrays')   
xlim(0,Rmax)
ylim(-Dmax,0)
grid(True)

figure(2)
stem(tau,abs(amp))
xlabel('Travel time (s)')
ylabel('Amplitude')
grid(True)

figure(3)
plot(tau,arrival_angle,'o')
xlabel('Travel time (s)')
ylabel('Arrival angle')
grid(True)

show()

print('done.')
